## $Id: studycourse.py 8745 2012-06-18 06:59:07Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container which holds the data of the student study courses
and contains the (student) study level objects.
"""
import grok
from zope.interface import implementedBy
from waeup.kofa.students.studycourse import (
    StudentStudyCourse, StudentStudyCourseFactory)
from waeup.kofa.utils.helpers import attrs_to_fields
from waeup.kofa.students.workflow import CLEARED, RETURNING, PAID
from waeup.fceokene.students.interfaces import (
    ICustomStudentStudyCourse, IStudentNavigation)

class CustomStudentStudyCourse(StudentStudyCourse):
    """This is a container for study levels.
    """

    grok.implements(ICustomStudentStudyCourse, IStudentNavigation)
    grok.provides(ICustomStudentStudyCourse)

    @property
    def next_session_allowed(self):
        state = self.student.state
        certificate = getattr(self, 'certificate', None)
        if certificate == None:
            return False
        if state == CLEARED:
            return True
        cond0 = state == RETURNING
        cond1 = self.previous_verdict in (
            'A','B','C','F','J','L','M','N','O','X','Z')
        cond2 = self.current_level in (0, 100)
        cond3 = certificate.study_mode.startswith('de') and \
            self.current_level == 200
        cond4 = certificate.study_mode.startswith('ph') and \
            self.current_level == 300
        if cond0 and (cond1 or cond2 or cond3 or cond4):
            return True
        cond5 = self.is_postgrad
        cond6 = state == PAID
        if  cond5 and cond6:
            return True
        return False

CustomStudentStudyCourse = attrs_to_fields(CustomStudentStudyCourse)

class CustomStudentStudyCourseFactory(StudentStudyCourseFactory):
    """A factory for student study courses.
    """

    def __call__(self, *args, **kw):
        return CustomStudentStudyCourse()

    def getInterfaces(self):
        return implementedBy(CustomStudentStudyCourse)
